@@ -43,6 +43,7 @@ extern __off64_t lseek64 __P ((int __fd, __off64_t __offset, int __whence));
 #include	<stdio.h>
 #include	<errno.h>
 #include	<string.h>
+#include	<syslog.h>
 #ifdef __dietlibc__NONO
 int strncmp(const char *s1, const char *s2, size_t n) __THROW __pure__;
 char *strncpy(char *dest, const char *src, size_t n) __THROW;
@@ -91,6 +92,7 @@ enum mode {
 };
 
 extern char short_options[];
+extern char short_bitmap_auto_options[];
 extern struct option long_options[];
 extern char Version[], Usage[], Help[], OptionHelp[],
 	Help_create[], Help_build[], Help_assemble[], Help_grow[],
@@ -158,7 +160,7 @@ struct mdstat_ent {
 	struct mdstat_ent *next;
 };
 
-extern struct mdstat_ent *mdstat_read(int);
+extern struct mdstat_ent *mdstat_read(int hold, int start);
 extern void free_mdstat(struct mdstat_ent *ms);
 extern void mdstat_wait(int seconds);
 
@@ -166,6 +168,8 @@ extern void mdstat_wait(int seconds);
 #define Sendmail "/usr/lib/sendmail -t"
 #endif
 
+#define SYSLOG_FACILITY LOG_DAEMON
+
 extern char *map_num(mapping_t *map, int num);
 extern int map_name(mapping_t *map, char *name);
 extern mapping_t r5layout[], pers[], modes[], faultylayout[];
@@ -182,7 +186,7 @@ extern struct superswitch {
 	void (*getinfo_super)(struct mdinfo *info, mddev_ident_t ident, void *sbv);
 	int (*update_super)(struct mdinfo *info, void *sbv, char *update, char *devname, int verbose);
 	__u64 (*event_super)(void *sbv);
-	int (*init_super)(struct supertype *st, void **sbp, mdu_array_info_t *info, char *name);
+	int (*init_super)(struct supertype *st, void **sbp, mdu_array_info_t *info, unsigned long long size, char *name);
 	void (*add_to_super)(void *sbv, mdu_disk_info_t *dinfo);
 	int (*store_super)(struct supertype *st, int fd, void *sbv);
 	int (*write_init_super)(struct supertype *st, void *sbv, mdu_disk_info_t *dinfo, char *devname);
@@ -190,8 +194,9 @@ extern struct superswitch {
 	int (*load_super)(struct supertype *st, int fd, void **sbp, char *devname);
 	struct supertype * (*match_metadata_desc)(char *arg);
 	__u64 (*avail_size)(struct supertype *st, __u64 size);
-	int (*add_internal_bitmap)(struct supertype *st, void *sbv, int chunk, int delay, int write_behind, int *sizep, int may_change);
-	void (*locate_bitmap)(struct supertype *st, int fd);
+	int (*add_internal_bitmap)(struct supertype *st, void *sbv, int chunk, int delay, int write_behind,
+				   unsigned long long size, int may_change, int major);
+	void (*locate_bitmap)(struct supertype *st, int fd, void *sbv);
 	int (*write_bitmap)(struct supertype *st, int fd, void *sbv);
 	int major;
 	int swapuuid; /* true if uuid is bigending rather than hostendian */
@@ -206,6 +211,10 @@ struct supertype {
 extern struct supertype *super_by_version(int vers, int minor);
 extern struct supertype *guess_super(int fd);
 
+#if __GNUC__ < 3
+struct stat64;
+#endif
+
 #ifdef UCLIBC
   struct FTW {};
 # define FTW_PHYS 1
@@ -227,7 +236,7 @@ extern int Manage_reconfig(char *devname, int fd, int layout);
 extern int Manage_subdevs(char *devname, int fd,
 			  mddev_dev_t devlist, int verbose);
 extern int Grow_Add_device(char *devname, int fd, char *newdev);
-extern int Grow_addbitmap(char *devname, int fd, char *file, int chunk, int delay, int write_behind);
+extern int Grow_addbitmap(char *devname, int fd, char *file, int chunk, int delay, int write_behind, int force);
 
 
 extern int Assemble(struct supertype *st, char *mddev, int mdfd,
@@ -245,10 +254,10 @@ extern int Build(char *mddev, int mdfd, int chunk, int level, int layout,
 
 
 extern int Create(struct supertype *st, char *mddev, int mdfd,
-		  int chunk, int level, int layout, unsigned long size, int raiddisks, int sparedisks,
+		  int chunk, int level, int layout, unsigned long long size, int raiddisks, int sparedisks,
 		  char *name,
 		  int subdevs, mddev_dev_t devlist,
-		  int runstop, int verbose, int force,
+		  int runstop, int verbose, int force, int assume_clean,
 		  char *bitmap_file, int bitmap_chunk, int write_behind, int delay);
 
 extern int Detail(char *dev, int brief, int test);
@@ -258,14 +267,15 @@ extern int Examine(mddev_dev_t devlist, int brief, int scan, int SparcAdjust,
 extern int Monitor(mddev_dev_t devlist,
 		   char *mailaddr, char *alert_cmd,
 		   int period, int daemonise, int scan, int oneshot,
-		   char *config, int test, char *pidfile);
+		   int dosyslog, char *config, int test, char *pidfile);
 
 extern int Kill(char *dev, int force);
 
 extern int CreateBitmap(char *filename, int force, char uuid[16],
 			unsigned long chunksize, unsigned long daemon_sleep,
 			unsigned long write_behind,
-			unsigned long long array_size);
+			unsigned long long array_size,
+			int major);
 extern int ExamineBitmap(char *filename, int brief, struct supertype *st);
 
 extern int md_get_version(int fd);
@@ -276,6 +286,7 @@ extern int check_reiser(int fd, char *name);
 extern int check_raid(int fd, char *name);
 
 extern int get_mdp_major(void);
+extern int dev_open(char *dev, int flags);
 extern int is_standard(char *dev, int *nump);
 
 
@@ -291,8 +302,10 @@ extern void uuid_from_super(int uuid[4], mdp_super_t *super);
 extern int same_uuid(int a[4], int b[4], int swapuuid);
 /* extern int compare_super(mdp_super_t *first, mdp_super_t *second);*/
 extern unsigned long calc_csum(void *super, int bytes);
-extern int enough(int level, int raid_disks, int avail_disks);
+extern int enough(int level, int raid_disks, int layout,
+		   char *avail, int avail_disks);
 extern int ask(char *mesg);
+extern unsigned long long get_component_size(int fd);
 
 
 extern char *human_size(long long bytes);
